function w = MSC(X,k,wp)

%Input: X (n x p, n instances and p dimensions), k the number of clusters,
%wp the previously obtained weight vectors/subspaces (p x m)

%Ouput: w=[wp,w_new]

%Copyright@Juhua Hu, May 12, 2016

[ins,dim] = size(X);

%Randomly initialize a weight vector in the simplex
w = rand(dim,1);
w = w./sum(w);

%Set the number of gradient ascent iterations, choose an appropriate one
T = 30;

%Set the parameter of tradoff between the gap and subspace differences
delta = 0.001;%choose from 1e-4:1
eta = 1; %step size


normPre = 9999;%norm used to determine step size updating
count = 1;
for i=1:T
    G = zeros(dim,1); %Gradient vector    
    tmpX = X.*repmat(w',ins,1); %X within subspace w 
    
    %Calculate the normalized Laplacian matrix
    S = squareform(pdist(tmpX)).^2;
    S = exp(-S)-eye(ins,ins); 
    d = sum(S,2);
    DD = 1./sqrt(d);    
    normD = DD*DD';
    SS = S.*normD;
    
    %Eigendecomposition
    [V,D] = eig((SS+SS')./2);
    dvalue = diag(D);
    [~,ind] = sort(dvalue,'descend');
    V = V(:,ind);
    Vk = V(:,k);
    Vk1 = V(:,k+1);
    
    %Calculate the gradient
    DD2 = 1./d.^(1.5);
    comp1 = DD2*DD'./2; 
    comp2 = DD*DD2'./2; 
    for j=1:dim
        tmpG = X(:,j);
        tmpd = squareform(pdist(tmpG)).^2; 
        tmpS = -(2*w(j)).*S.*tmpd;
        dd = sum(tmpS,2);
        tmpS1 = tmpS.*normD;
        comp3 = repmat(dd',ins,1).*comp1+repmat(dd,1,ins).*comp2;
        tmpS2 = comp3.*S;
        G(j) = (Vk'*(tmpS1-tmpS2))*Vk - (Vk1'*(tmpS1-tmpS2))*Vk1;
    end
    
    %Update gradient when having previous solutions
    if ~isempty(wp)
        num_P=size(wp,2);
        G=G+(delta/num_P).*sum(repmat(w,1,num_P)-wp,2);
    end

    %Decrease the step size if needed
    normCur = norm(G);
    if normCur>normPre
        count = count+1;
        eta = 1/count;
    end
    normPre = normCur;
    
    % Gradient ascent
    w = w+eta.*G;
    
    %Project to the simplex
    w_Sort=sort(w,'descend');
    for mm=sort(1:dim,'descend')
        if w_Sort(mm)>(1/mm)*(sum(w_Sort(1:mm))-1)
            break;
        end
    end
    tau=(1/mm)*(sum(w_Sort(1:mm))-1);
    w=w-tau;
    w(w<0)=0;
    
end

%return all w's
w = [wp,w];
end