%Demo
%Copyright@Juhua Hu, Aug 10, 2017

%Clean as needed
clear;
clc;

%Track the random seed
stream = RandStream.getGlobalStream;
reset(stream);

%Load data
X=rand(50,2); %X (n x p, n instances and p dimensions)
reset(stream);
[ins,dim] = size(X);


%Normalize the data if needed
 for i=1:size(X,1)
     if norm(X(i,:))>0
         X(i,:) = X(i,:)./norm(X(i,:));
     end
 end

%Select representatives for large-scale data (>10,000)
repX = X; % X used for finding subspaces
I = 1:ins; %index of instances used for finding subspaces
if ins>10000   
    %Randomly select 2000 representatives 
    I = randperm(ins);
    I = I(1:2000);
    repX = X(I,:);
end
restI = 1:ins; %index of rest instances
restI(I)=[];
reset(stream);

%Determine the number of clusters (largest eigengap in original space) if needed
w = ones(dim,1)./dim;
tmpX = repX.*repmat(w',length(I),1);
S = squareform(pdist(tmpX)).^2; %Calculate Laplacian
S = exp(-S); 
d = sum(S,2);
DD = 1./sqrt(d);
normD = DD*DD';
SS = S.*normD;
[V,D] = eig((SS+SS')./2); % Eig decomposition
dvalue = diag(D);
[vv,~] = sort(dvalue,'descend');
gaps = vv(2:length(I)-1)-vv(3:length(I)); % Eigengaps
[~,ind] = max(gaps);
k = ind+1;
%Otherwise set the number of clusters
%k = 2;

%initialize solution set
w = [];
count3 = 0; %determine the termination of searching
while count3<3 %3 is a parameter (3 times of no new solution found)
    w = MSC(repX,k,w);  
    if size(w,2)>1
        dis = pdist(w').^2; %Calculate distances between w's
        if ~isempty(find(dis<=0.1, 1)) % HERE 0.1 IS A PARAMETER TO CHOOSE
            w=w(:,1:size(w,2)-1);         
            count3 = count3+1;
        else
            count3=0;
        end
    else
        count3=0;
    end
end

%Use spectral clustering to find clusters in each subspace
label = cell(size(w,2),1);
for i=1:size(w,2)   
    curW = w(:,i);
    tmpX = repX.*repmat(curW',length(I),1);
    [label{i}(I),vCenters,tmp,tmpDs] = spclustering(tmpX,k,stream);
    [tmpM,tmpI]=min(tmpDs);
    centers = tmpX(tmpI,:);
    %Determine the lable of rest data if large-scale
    if length(I)<ins
        restX = X(restI,:).*repmat(curW',length(restI),1);
        d2c = repmat(sum(restX.^2,2),1,size(centers,1))+repmat(sum(centers.^2,2)',size(restX,1),1)-2.*restX*centers';
        [minV,minI] = min(d2c,[],2);
        label{i}(restI) = minI;
    end 
end